module core2175_sim
(
test_pat,

fmc_fr,
fmc6x1_clk,   fmc7x1_clk,   fmc8x1_clk,
fmc6x2_clk,   fmc7x2_clk,   fmc8x2_clk,

shdn,
mode,

outa,
outb

);
input [15:0] test_pat;

input fmc_fr;

input fmc6x1_clk,   fmc7x1_clk,   fmc8x1_clk;
input fmc6x2_clk,   fmc7x2_clk,   fmc8x2_clk;

input shdn;
input [2:0] mode;

output outa;
output outb;

wire [11:0]  pda_in_6x1;
wire [13:0]  pda_in_7x1;
wire [15:0]  pda_in_8x1;
wire [7:0]   pda_in_8x21;
wire [7:0]   pda_in_8x20;
wire [13:0]   pda_in_7x21;
wire [13:0]   pda_in_7x20;
wire [5:0]   pda_in_6x21;
wire [5:0]   pda_in_6x20;

wire       sda_out_6x1;
wire       sda_out_7x1;
wire       sda_out_8x1;
wire       sda_out_8x21;
wire       sda_out_8x20;
wire       sda_out_7x21;
wire       sda_out_7x20;
wire       sda_out_6x21;
wire       sda_out_6x20;


serializer_sim #
(
     .SERIAL_BITS              (12)
)
serializer_6x1
(
.pda_in(pda_in_6x1),
.sda_out(sda_out_6x1),
.clk1x(fmc_fr),
.clkxx(fmc6x1_clk),
.rst(shdn)
);

assign pda_in_6x1 = test_pat[11:0];



serializer_sim #
(
     .SERIAL_BITS              (14)
)
serializer_7x1
(
.pda_in(pda_in_7x1),
.sda_out(sda_out_7x1),
.clk1x(fmc_fr),
.clkxx(fmc7x1_clk),
.rst(shdn)
);

assign pda_in_7x1 = test_pat[13:0];


serializer_sim #
(
     .SERIAL_BITS              (16)
)
serializer_8x1
(
.pda_in(pda_in_8x1),
.sda_out(sda_out_8x1),
.clk1x(fmc_fr),
.clkxx(fmc8x1_clk),
.rst(shdn)
);

assign pda_in_8x1 = test_pat[15:0];



serializer_sim #
(
     .SERIAL_BITS              (6)
)
serializer_6x21
(
.pda_in(pda_in_6x21),
.sda_out(sda_out_6x21),
.clk1x(fmc_fr),
.clkxx(fmc6x2_clk),
.rst(shdn)
);
serializer_sim #
(
     .SERIAL_BITS              (6)
)
serializer_6x20
(
.pda_in(pda_in_6x20),
.sda_out(sda_out_6x20),
.clk1x(fmc_fr),
.clkxx(fmc6x2_clk),
.rst(shdn)
);


assign pda_in_6x21 = {test_pat[11], test_pat[9], test_pat[7], test_pat[5], test_pat[3], test_pat[1]};
assign pda_in_6x20 = {test_pat[10], test_pat[8], test_pat[6], test_pat[4], test_pat[2], test_pat[0]};

reg fmc_fr_7x2;
reg [15:0] test_pat_14bit;
initial fmc_fr_7x2 = 1'b0;
always @ (posedge fmc_fr)
begin
  fmc_fr_7x2      <= ~fmc_fr_7x2;
  test_pat_14bit <= test_pat;  
end

reg [31:0] test_pat_28bit;
always @ (posedge fmc_fr)
  if(~fmc_fr_7x2)
    test_pat_28bit <= {test_pat_14bit, test_pat};    

serializer_sim #
(
     .SERIAL_BITS              (14)
)
serializer_7x21
(
.pda_in(pda_in_7x21),
.sda_out(sda_out_7x21),
.clk1x(fmc_fr_7x2),
.clkxx(fmc7x2_clk),
.rst(shdn)
);
serializer_sim #
(
     .SERIAL_BITS              (14)
)
serializer_7x20
(
.pda_in(pda_in_7x20),
.sda_out(sda_out_7x20),
.clk1x(fmc_fr_7x2),
.clkxx(fmc7x2_clk),
.rst(shdn)
);

assign pda_in_7x21 = {test_pat_28bit[29], test_pat_28bit[27], test_pat_28bit[25], test_pat_28bit[23], test_pat_28bit[21], test_pat_28bit[19], test_pat_28bit[17],
                      test_pat_28bit[13], test_pat_28bit[11], test_pat_28bit[9],  test_pat_28bit[7],  test_pat_28bit[5],  test_pat_28bit[3],  test_pat_28bit[1]};
                      
assign pda_in_7x20 = {test_pat_28bit[28], test_pat_28bit[26], test_pat_28bit[24], test_pat_28bit[22], test_pat_28bit[20], test_pat_28bit[18], test_pat_28bit[16],
                      test_pat_28bit[12], test_pat_28bit[10], test_pat_28bit[8],  test_pat_28bit[6],  test_pat_28bit[4],  test_pat_28bit[2],  test_pat_28bit[0]};
  
  

serializer_sim #
(
     .SERIAL_BITS              (8)
)
serializer_8x21
(
.pda_in(pda_in_8x21),
.sda_out(sda_out_8x21),
.clk1x(fmc_fr),
.clkxx(fmc8x2_clk),
.rst(shdn)
);
serializer_sim #
(
     .SERIAL_BITS              (8)
)
serializer_8x20
(
.pda_in(pda_in_8x20),
.sda_out(sda_out_8x20),
.clk1x(fmc_fr),
.clkxx(fmc8x2_clk),
.rst(shdn)
);


assign pda_in_8x21 = {test_pat[15], test_pat[13], test_pat[11], test_pat[9], test_pat[7], test_pat[5], test_pat[3], test_pat[1]};
assign pda_in_8x20 = {test_pat[14], test_pat[12], test_pat[10], test_pat[8], test_pat[6], test_pat[4], test_pat[2], test_pat[0]};


reg outa;
reg outb;

always @ (mode or 
          shdn or
          
          sda_out_6x1 or
          sda_out_7x1 or
          sda_out_8x1 or
          sda_out_8x21 or
          sda_out_8x20 or
          sda_out_7x21 or
          sda_out_7x20 or
          sda_out_6x21 or
          sda_out_6x20)
begin
  if (~shdn)
  begin
    outa  <= 1'b0;
    outb  <= 1'b0;
  end
  else
  begin
  case (mode)
  3'b000:begin
               outa  <= sda_out_8x21;
               outb  <= sda_out_8x20;
             end   
  3'b001:begin
               outa  <= sda_out_7x21;
               outb  <= sda_out_7x20;
             end  
  3'b010:begin
               outa  <= sda_out_6x21;
               outb  <= sda_out_6x20;
             end  
  3'b011:begin
               outa  <= 1'b0;
               outb  <= 1'b0;
             end   
  3'b100:begin
               outa  <= 1'b0;
               outb  <= 1'b0;
             end   
  3'b101:begin
               outa  <= sda_out_7x1;
               outb  <= 1'b0;
             end  
  3'b110:begin
               outa  <= sda_out_6x1;
               outb  <= 1'b0;
             end  
  3'b111:begin
               outa  <= sda_out_8x1;
               outb  <= 1'b0;
             end  
  default:begin
               outa  <= 1'b0;
               outb  <= 1'b0;
             end  
  endcase
  end
end  

endmodule
