/******************************************************************************
* Copyright (c) 2011, Linear Technology, Corp.                                *
* Linear Tech Proprietary and Confidential                                    *
*                                                                             *
* Title:                                                                      *
* Author:          Gary Yu                                                    *
* Create Date:     10:08:58 09/09/2011                                        *
* Design Name:                                                                *
* Module Name:                                                                *
* Project Name:                                                               *
* Target Devices:                                                             *
* Tool versions:                                                              *
* Description:                                                                *
*                                                                             *
* Dependencies:                                                               *
*                                                                             *
* Revision 1.0     10:08:58 09/09/2011 Gary Yu                                *
* File Created                                                                *
*                                                                             *
* Additional Comments:                                                        *
*                                                                             *
******************************************************************************/

`timescale 1ns / 1ps

module altbram_ctrl(
capture_start,
collection_start,

capture_ref,
collection_ref,

capture_done,
capture_display,
collection_display,

cllctrl,
clk,
dclk,

rst,

bram_add_ctrl,
bram_clk_ctrl,
bram_wren

  );
input capture_start;
input collection_start;

input capture_ref;
input collection_ref;

input capture_done;
output capture_display;
output collection_display;

output cllctrl;
input clk;
input dclk;

input rst;

output bram_add_ctrl;
output bram_clk_ctrl;
output bram_wren;

reg       bram_add_ctrl;
reg       bram_clk_ctrl;
reg       bram_wren;
reg [6:0] bram_ctrl_reg;
reg [3:0] bram_done_reg;
reg       bram_wr, bram_rd;

reg       capture_display;
reg       collection_display;

always @ (posedge clk or negedge rst)
  if(~rst)
    bram_ctrl_reg[6:0] <= #1 7'b0;
  else
  begin
    bram_ctrl_reg[0] <= #1 capture_start | collection_start;
    bram_ctrl_reg[1] <= #1 bram_ctrl_reg[0];
    bram_ctrl_reg[2] <= #1 bram_ctrl_reg[1];
    bram_ctrl_reg[3] <= #1 bram_ctrl_reg[2];
    bram_ctrl_reg[4] <= #1 bram_ctrl_reg[3];
    bram_ctrl_reg[5] <= #1 bram_ctrl_reg[4];
    bram_ctrl_reg[6] <= #1 bram_ctrl_reg[5];
  end

always @ (posedge clk or negedge rst)
  if(~rst)
  begin
    bram_wr          <= #1 1'b0; 
    bram_rd          <= #1 1'b0;
  end
  else if (capture_start)
  begin
    bram_wr          <= #1 1'b1; 
    bram_rd          <= #1 1'b0;
  end
  else if (collection_start)
  begin
    bram_wr          <= #1 1'b0; 
    bram_rd          <= #1 1'b1;
  end
  
      
always @ (posedge clk or negedge rst)
  if(~rst)
    bram_done_reg[3:0] <= #1 4'b0;
  else
  begin
    bram_done_reg[0] <= #1 capture_done;
    bram_done_reg[1] <= #1 bram_done_reg[0];
    bram_done_reg[2] <= #1 bram_done_reg[1];
    bram_done_reg[3] <= #1 bram_done_reg[2];
   end

  

always @ (posedge clk or negedge rst)
  if(~rst)
    bram_add_ctrl      <= #1 1'b0;
  else if (bram_ctrl_reg[5])
    bram_add_ctrl      <= #1 1'b1;
  else if (bram_done_reg[0])
    bram_add_ctrl      <= #1 1'b0;

    
always @ (posedge clk or negedge rst)
  if(~rst)
  begin
    bram_clk_ctrl      <= #1 1'b0;
    bram_wren          <= #1 1'b0;
  end
  else if (bram_ctrl_reg[6] & bram_wr)
  begin
    bram_clk_ctrl      <= #1 1'b1;
    bram_wren          <= #1 1'b1;
  end
  else if (bram_ctrl_reg[6] & bram_rd)
  begin
    bram_clk_ctrl      <= #1 1'b1;
    bram_wren          <= #1 1'b0;
  end
  else if (capture_done & bram_wr)
  begin
    bram_clk_ctrl      <= #1 1'b0;
    bram_wren          <= #1 1'b0;
  end
  else if (bram_done_reg[0] & bram_rd)
  begin
    bram_clk_ctrl      <= #1 1'b0;
    bram_wren          <= #1 1'b0;
  end

always @ (posedge clk or negedge rst)
  if(~rst)
  begin
    capture_display     <= #1 1'b0;
    collection_display  <= #1 1'b0;
  end  
  else if (bram_done_reg[3] & capture_ref)
  begin
    capture_display     <= #1 1'b1;
    collection_display  <= #1 1'b0;
  end
  else if (bram_done_reg[3] & collection_ref)
  begin
    capture_display     <= #1 1'b0;
    collection_display  <= #1 1'b1;
  end
 
    
reg cllctrl;  
reg [3:0] cllctrl_reg;
always @ (posedge dclk or negedge rst)
if (~rst)
begin
 cllctrl     <= #1 1'b0;  
 cllctrl_reg <= #1 4'b0;
end
else
begin if (bram_rd)  
 cllctrl_reg[0] <= #1 bram_clk_ctrl;
 cllctrl_reg[1] <= #1 cllctrl_reg[0];
 cllctrl_reg[2] <= #1 cllctrl_reg[1];
 cllctrl_reg[3] <= #1 cllctrl_reg[2];
 cllctrl        <= #1 cllctrl_reg[0] & cllctrl_reg[1];
end
    
endmodule

