/******************************************************************************
* Copyright (c) 2011, Linear Technology, Corp.                                *
* Linear Tech Proprietary and Confidential                                    *
*                                                                             *
* Title:                                                                      *
* Author:          Gary Yu                                                    *
* Create Date:     10:08:58 09/09/2011                                        *
* Design Name:                                                                *
* Module Name:                                                                *
* Project Name:                                                               *
* Target Devices:                                                             *
* Tool versions:                                                              *
* Description:                                                                *
*                                                                             *
* Dependencies:                                                               *
*                                                                             *
* Revision 1.0     10:08:58 09/09/2011 Gary Yu                                *
* File Created                                                                *
*                                                                             *
* Additional Comments: based on 50Mhz     parameter PBDELAY = 650000          *
*                                                                             *
******************************************************************************/
module debounce (reset, clk, bounce_in, clear_out);
   input reset, clk, bounce_in;
   output clear_out;

   //parameter PBDELAY = 650000;
   parameter PBDELAY = 8;
   parameter PBBITS  = 20;

   reg [PBBITS-1:0] count;
   reg pb_buf, clear_out;
   
   wire debflag = bounce_in ^ pb_buf;
   always @(posedge clk or negedge reset)
     if (~reset) 
     begin 
       pb_buf    <= #1 bounce_in; 
       clear_out <= #1 bounce_in; 
       count     <= #1 0; 
     end
     else if (debflag) 
     begin 
       pb_buf    <= #1 bounce_in; 
       count     <= #1 0; 
     end
     else if (count == PBDELAY) 
       clear_out <= #1 pb_buf;
     else 
       count     <= #1 count + 1;

endmodule
