/******************************************************************************
* Copyright (c) 2011, Linear Technology, Corp.                                *
* Linear Tech Proprietary and Confidential                                    *
*                                                                             *
* Title:                                                                      *
* Author:          Gary Yu                                                    *
* Create Date:     10:08:58 09/09/2011                                        *
* Design Name:                                                                *
* Module Name:                                                                *
* Project Name:                                                               *
* Target Devices:                                                             *
* Tool versions:                                                              *
* Description:                                                                *
*                                                                             *
* Dependencies:                                                               *
*                                                                             *
* Revision 1.0     10:08:58 09/09/2011 Gary Yu                                *
* File Created                                                                *
*                                                                             *
* Additional Comments:                                                        *
*                                                                             *
******************************************************************************/

`timescale 1ns / 1ps

module top_FPGA
(
  // ----------------------------------------------------
  // System Clock 
  // ----------------------------------------------------
    SYSCLK,
    
  // ----------------------------------------------------
  // LCD interface 
  // ----------------------------------------------------

    lcd_d_cn,
    lcd_wen,
    lcd_csn,
    lcd_data,

  // ----------------------------------------------------
  // General Control and Jumper 
  // ----------------------------------------------------

//MPI2_RESET,
//MODE_JMP,
//MEMSIZE_JMP,
//CHSEL_JMP,
  // ----------------------------------------------------
  // 9011 interface
  // ----------------------------------------------------
//capture_start,  
//data_collection,

clldata,
cllclk,
cllctrl,

dco_u_p,
dco_u_n,

fr_u_p,
fr_u_n,

out0a_p,
out0a_n,

out0b_p,
out0b_n,

out1a_p,
out1a_n,

out1b_p,
out1b_n,

out2a_p,
out2a_n,

out2b_p,
out2b_n,

out3a_p,
out3a_n,

out3b_p,
out3b_n,

dco_d_p,
dco_d_n,

fr_d_p,
fr_d_n,

out4a_p,
out4a_n,

out4b_p,
out4b_n,

out5a_p,
out5a_n,

out5b_p,
out5b_n,

out6a_p,
out6a_n,

out6b_p,
out6b_n,

out7a_p,
out7a_n,

out7b_p,
out7b_n//,
  
  // ----------------------------------------------------
  // Indication
  // ----------------------------------------------------
  
    //CDA_MAX,//gyu test
    //DPA_LOCK,//gyu test   
    //CAPTURE_DONE,
    //TILE0_PLLLKDET_OUT
     
);

`include "../src/fpga/char_define.h"
//***********************************Ports Declaration*******************************

  // ----------------------------------------------------
  // System Clock 
  // ----------------------------------------------------
input       SYSCLK;
    
  // ----------------------------------------------------
  // LCD interface 
  // ----------------------------------------------------

output      lcd_d_cn;   //LCD Data/Command select
output      lcd_wen;    //LCD Read Write select
output      lcd_csn;    //LCD Chip select
inout [7:0] lcd_data;   //LCD Data bus
 
  // ----------------------------------------------------
  // General Control and Jumper 
  // ----------------------------------------------------
  
wire MPI2_RESET;
//input MODE_JMP;
//input [3:0] MEMSIZE_JMP;
//input [2:0] CHSEL_JMP; 
  // ----------------------------------------------------
  // 9011 interface
  // ----------------------------------------------------
//input capture_start;  
//input data_collection;
  
output [15:0] clldata;
output        cllclk;
output        cllctrl;

input dco_u_p;
input dco_u_n;

input fr_u_p;
input fr_u_n;

input out0a_p;
input out0a_n;

input out0b_p;
input out0b_n;

input out1a_p;
input out1a_n;

input out1b_p;
input out1b_n;

input out2a_p;
input out2a_n;

input out2b_p;
input out2b_n;

input out3a_p;
input out3a_n;

input out3b_p;
input out3b_n;


input dco_d_p;
input dco_d_n;

input fr_d_p;
input fr_d_n;

input out4a_p;
input out4a_n;

input out4b_p;
input out4b_n;

input out5a_p;
input out5a_n;

input out5b_p;
input out5b_n;

input out6a_p;
input out6a_n;

input out6b_p;
input out6b_n;

input out7a_p;
input out7a_n;

input out7b_p;
input out7b_n;
    
  // ----------------------------------------------------
  // Indication
  // ----------------------------------------------------

//output CDA_MAX;//gyu test
//output [9:0] DPA_LOCK;//gyu test
//output CAPTURE_DONE;
//output TILE0_PLLLKDET_OUT;

     
//************************** Register Declarations ****************************

//**************************** Wire Declarations ******************************

      
  // ----------------------------------------------------
  // 9011 interface
  // ----------------------------------------------------
 
wire dco_u_gen;

wire fr_u_gen;

wire out0a_gen;
wire out0b_gen;
wire out1a_gen;
wire out1b_gen;
wire out2a_gen;
wire out2b_gen;
wire out3a_gen;
wire out3b_gen;
 
wire dco_d_gen;

wire fr_d_gen;

wire out4a_gen;
wire out4b_gen;
wire out5a_gen;
wire out5b_gen;
wire out6a_gen;
wire out6b_gen;
wire out7a_gen;
wire out7b_gen;

      
//**************************** Main Body of Code *******************************


    
  // ----------------------------------------------------
  // 9011 interface
  // ----------------------------------------------------

    alt_inbuf_diff clk_buf_dco_u
    (
        .o                              (dco_u_gen),
        .i                              (dco_u_p),
        .ibar                           (dco_u_n)
    );    
      
    alt_inbuf_diff fr_u_ibufds_i 
    (
        .o                              (fr_u_gen),
        .i                              (fr_u_p),
        .ibar                           (fr_u_n)
    );

    alt_inbuf_diff out0a_ibufds_i 
    (
        .o                              (out0a_gen),
        .i                              (out0a_p),
        .ibar                           (out0a_n)
    );

    alt_inbuf_diff out0b_ibufds_i  
    (
        .o                              (out0b_gen),
        .i                              (out0b_p),
        .ibar                           (out0b_n)
    );
    
    alt_inbuf_diff out1a_ibufds_i
    (
        .o                              (out1a_gen),
        .i                              (out1a_p),
        .ibar                           (out1a_n)
    );

    alt_inbuf_diff out1b_ibufds_i 
    (
        .o                              (out1b_gen),
        .i                              (out1b_p),
        .ibar                           (out1b_n)
    );
 
    alt_inbuf_diff out2a_ibufds_i  
    (
        .o                              (out2a_gen), 
        .i                              (out2a_p),
        .ibar                           (out2a_n)
    );

    alt_inbuf_diff out2b_ibufds_i 
    (
        .o                              (out2b_gen),
        .i                              (out2b_p),
        .ibar                           (out2b_n)
    );

    alt_inbuf_diff out3a_ibufds_i 
    (
        .o                              (out3a_gen),
        .i                              (out3a_p),
        .ibar                           (out3a_n)
    );

    alt_inbuf_diff out3b_ibufds_i 
    (
        .o                              (out3b_gen),
        .i                              (out3b_p),
        .ibar                           (out3b_n)
    );


    alt_inbuf_diff clk_buf_dco_d
    (
        .o                              (dco_d_gen),
        .i                              (dco_d_p),
        .ibar                           (dco_d_n)
    );    
  
    alt_inbuf_diff fr_d_ibufds_i 
    (
        .o                              (fr_d_gen),
        .i                              (fr_d_p),
        .ibar                           (fr_d_n)
    );

    alt_inbuf_diff out4a_ibufds_i
    (
        .o                              (out4a_gen),
        .i                              (out4a_p),
        .ibar                           (out4a_n)
    );

    alt_inbuf_diff out4b_ibufds_i  
    (
        .o                              (out4b_gen),
        .i                              (out4b_p),
        .ibar                           (out4b_n)
    );
   
    alt_inbuf_diff out5a_ibufds_i 
    (
        .o                              (out5a_gen),
        .i                              (out5a_p),
        .ibar                           (out5a_n)
    );

    alt_inbuf_diff out5b_ibufds_i 
    (
        .o                              (out5b_gen),
        .i                              (out5b_p),
        .ibar                           (out5b_n)
    );

    alt_inbuf_diff out6a_ibufds_i  
    (
        .o                              (out6a_gen), 
        .i                              (out6a_p),
        .ibar                           (out6a_n)
    );

    alt_inbuf_diff out6b_ibufds_i 
    (
        .o                              (out6b_gen),
        .i                              (out6b_p),
        .ibar                           (out6b_n)
    );

    alt_inbuf_diff out7a_ibufds_i
    (
        .o                              (out7a_gen),
        .i                              (out7a_p),
        .ibar                           (out7a_n)
    );

    alt_inbuf_diff out7b_ibufds_i
    (
        .o                              (out7b_gen),
        .i                              (out7b_p),
        .ibar                           (out7b_n)
    );
    
wire [3:0] memsize;
wire [2:0] chsel;
    
    
wire sclk, cclk;
wire rx_inclk, rx_outclk, pre_clk, othclk, cllclk, clldclk;
wire pll2_locked, rx_locked, pll4_locked;

reg  rx_locked_reg;
reg  pll4_locked_reg;

altpll_source2 altpll_source2(
.inclk0(SYSCLK),
.c0(sclk),//250K
.c1(cclk),//5M
.locked(pll2_locked));

  
altpll_source4 altpll_source4(
       .areset((~pll2_locked) | (~MPI2_RESET)),//pll reset

       .inclk0(dco_u_gen),
       .inclk1(dco_d_gen),

       .c0(pre_clk),
       .c1(cllclk),
       .c2(othclk),
       .c3(clldclk),
       .c4(rx_inclk),
       .locked(pll4_locked));

always @ (posedge othclk or negedge MPI2_RESET)                                               
  if (~MPI2_RESET)
    pll4_locked_reg <= #1 1'b0;
  else
    pll4_locked_reg <= #1 pll4_locked;  
       
wire  [17:0]   rx_dpa_locked;

wire           pll_areset;
reg    [17:0]  rx_reset;
reg    [17:0]  rx_cda_reset;
reg            bitslip_reset;
            
wire   [17:0]  rx_dpll_hold;
      
wire   [17:0]  rx_cda_max;
wire   [143:0] rx_out;
      
wire bsp_clr = 1'b0;
wire bsp_start = 1'b1;
wire bsp_gen;
wire bsp_cmp;
wire bsp_err;

reg  rx_dpa_locked_reg;

assign #1 rx_dpll_hold = {18{bsp_cmp}};
assign #1 pll_areset   = (~rx_locked_reg) | (~MPI2_RESET);


reg [2:0] rx_reset_cnt_reg;
always @ (posedge othclk or posedge pll_areset)  
  if (pll_areset)
  begin
    rx_reset_cnt_reg <= #1 3'b0; 
    rx_reset         <= #1 18'h3ffff;
  end
  else if (rx_reset_cnt_reg == 3'b011)
  begin
    rx_reset_cnt_reg <= #1 rx_reset_cnt_reg + 1; 
    rx_reset         <= #1 18'b0;
  end
  else if (rx_reset_cnt_reg == 3'b111)
  begin
    rx_reset_cnt_reg <= #1 rx_reset_cnt_reg;
    rx_reset         <= #1 rx_reset; 
  end
  else
  begin
    rx_reset_cnt_reg <= #1 rx_reset_cnt_reg + 1;
    rx_reset         <= #1 rx_reset; 
  end


  
reg [4:0] rx_cda_reset_cnt_reg;
always @ (posedge othclk or posedge pll_areset)  
  if (pll_areset)
  begin
    rx_cda_reset_cnt_reg <= #1 5'b0; 
    rx_cda_reset         <= #1 18'b0;
    bitslip_reset        <= #1 1'b0;
  end
  else if (rx_dpa_locked_reg)
  begin
    if(rx_cda_reset_cnt_reg == 5'h03)
    begin
              rx_cda_reset_cnt_reg <= #1 rx_cda_reset_cnt_reg + 1;
              rx_cda_reset         <= #1 18'h3ffff;
              bitslip_reset        <= #1 1'b0;
    end
    else if(rx_cda_reset_cnt_reg == 5'h10)
    begin
              rx_cda_reset_cnt_reg <= #1 rx_cda_reset_cnt_reg + 1;
              rx_cda_reset         <= #1 18'h0;
              bitslip_reset        <= #1 1'b0;
    end
    else if(rx_cda_reset_cnt_reg == 5'h16)
    begin
              rx_cda_reset_cnt_reg <= #1 rx_cda_reset_cnt_reg + 1;
              rx_cda_reset         <= #1 18'h0;
              bitslip_reset        <= #1 1'b1;
    end
    else if(rx_cda_reset_cnt_reg == 5'h1f)
    begin    
              rx_cda_reset_cnt_reg <= #1 rx_cda_reset_cnt_reg;
              rx_cda_reset         <= #1 rx_cda_reset;
              bitslip_reset        <= #1 bitslip_reset;
    end
    else
    begin
              rx_cda_reset_cnt_reg <= #1 rx_cda_reset_cnt_reg + 1;
              rx_cda_reset         <= #1 rx_cda_reset;
              bitslip_reset        <= #1 bitslip_reset;
    end
  end

       
always @ (posedge othclk or posedge pll_areset)  
  if (pll_areset)
    rx_dpa_locked_reg <= #1 1'b0;
  else
    rx_dpa_locked_reg <= #1  rx_dpa_locked[17] &
                             rx_dpa_locked[16] &
                             rx_dpa_locked[15] &
                             rx_dpa_locked[14] &
                             //rx_dpa_locked[13] &
                             //rx_dpa_locked[12] &
                             rx_dpa_locked[11] &
                             rx_dpa_locked[10] &
                             rx_dpa_locked[9] &
                             rx_dpa_locked[8] &
                             rx_dpa_locked[7] &
                             rx_dpa_locked[6] &
                             rx_dpa_locked[5] &
                             rx_dpa_locked[4] &
                             rx_dpa_locked[3] &
                             rx_dpa_locked[2] &
                             rx_dpa_locked[1] &
                             rx_dpa_locked[0]; 


altlvds_rx1 altlvds_rx1(
       .pll_areset(~pll4_locked_reg),
       .rx_reset(rx_reset),//dpa reset
       .rx_cda_reset(rx_cda_reset),//cda reset
       
       .rx_channel_data_align({18{bsp_gen}}),
       .rx_dpll_hold(rx_dpll_hold),
       .rx_in({fr_d_gen,
               fr_u_gen,
               
               out7a_gen,
               out7b_gen,
               out6a_gen,
               out6b_gen,
               out5a_gen,
               out5b_gen,
               out4a_gen,
               out4b_gen,
 
               out3a_gen,
               out3b_gen,
               out2a_gen,
               out2b_gen,
               out1a_gen,
               out1b_gen,
               out0a_gen,
               out0b_gen}),
       .rx_inclock(rx_inclk),
              
       .rx_cda_max(rx_cda_max),
       .rx_dpa_locked(rx_dpa_locked),//dpa locked
       .rx_out(rx_out),
       .rx_locked(rx_locked),
       .rx_outclock(rx_outclk));

always @ (posedge rx_outclk or negedge MPI2_RESET)                                               
  if (~MPI2_RESET)
    rx_locked_reg <= #1 1'b0;
  else
    rx_locked_reg <= #1 rx_locked;  
              
                          
reg [15:0] test_longf;
                            
bitslip_gen bitslip_gen(
.fclk(othclk),
.rst(bitslip_reset),
.bsp_clr(bsp_clr),
.bsp_start(bsp_start),
.data_in(test_longf),
.bsp_gen(bsp_gen),
.bsp_cmp(bsp_cmp),
.bsp_err(bsp_err)
);

wire  gtp_rst;
assign #1 gtp_rst = rx_locked_reg & pll2_locked & pll4_locked_reg & bsp_cmp;

wire capture_clr_start;
wire data_clr_collection;


reg [3:0] capture_reg;
reg capture_trig_start;
reg capture_ref;
always @ (posedge othclk or negedge gtp_rst)
  if (~gtp_rst)
  begin
    capture_reg            <= #1 4'h0;
    capture_ref            <= #1 1'b0;
    capture_trig_start     <= #1 1'b0;
  end  
  else
  begin
    capture_reg[0]         <= #1 capture_clr_start;
    capture_reg[1]         <= #1 capture_reg[0];
    capture_reg[2]         <= #1 capture_reg[1];
    capture_reg[3]         <= #1 capture_reg[2];
    capture_ref            <= #1 capture_reg[2];
    capture_trig_start     <= #1 (~capture_reg[3]) & capture_reg[2]; 
  end  

reg [3:0] collection_reg;
reg collection_trig_start;
reg collection_ref;
always @ (posedge othclk or negedge gtp_rst)
  if (~gtp_rst)
  begin
    collection_reg         <= #1 4'h0;
    collection_ref         <= #1 1'b0;
    collection_trig_start  <= #1 1'b0;
  end  
  else
  begin
    collection_reg[0]      <= #1 data_clr_collection;
    collection_reg[1]      <= #1 collection_reg[0];
    collection_reg[2]      <= #1 collection_reg[1];
    collection_reg[3]      <= #1 collection_reg[2];
    collection_ref         <= #1 collection_reg[2];
    collection_trig_start  <= #1 (~collection_reg[3]) & collection_reg[2]; 
  end  

    
wire bram_clk_ctrl;
wire bram_clk;
altclkctrl1 altclkctrl1(
.ena(bram_clk_ctrl),
.inclk(pre_clk),
.outclk(bram_clk));


wire bram_add_ctrl;
wire capture_done;
wire [13:0] bram_addr;
reg [10:0] BUFFER_ADDR_CNT;

altbram_addr altbram_addr
(
.addr_trigger(bram_clk), 
.addr(bram_addr),
.capture_done(capture_done),
.capture_data(bram_add_ctrl),
.BUFFER_ADDR_CNT({BUFFER_ADDR_CNT}));
defparam altbram_addr.ADDBITS = 14;

wire capture_display, collection_display;
altbram_ctrl altbram_ctrl
(
.capture_start(capture_trig_start),
.collection_start(collection_trig_start),
.capture_ref(capture_ref),
.collection_ref(collection_ref),

.capture_done(capture_done),
.capture_display(capture_display),
.collection_display(collection_display),
.cllctrl(cllctrl),
.clk(pre_clk),
.dclk(clldclk),

.rst(gtp_rst),

.bram_add_ctrl(bram_add_ctrl),
.bram_clk_ctrl(bram_clk_ctrl),
.bram_wren(bram_wren));


reg [15:0] test_long0, test_long1, test_long2, test_long3;
reg [15:0] test_long4, test_long5, test_long6, test_long7;

wire [127:0] test_long;       
altmem_ram2 altmem_ram2(
       .address(bram_addr),
       .clock(bram_clk),
       .data({test_long7, test_long6, test_long5, test_long4,
              test_long3, test_long2, test_long1, test_long0}),
       .wren(bram_wren),
       .q(test_long));
       
                                 
reg [15:0] clldata;
always @ (posedge clldclk or negedge gtp_rst)
  if (~gtp_rst)
    clldata    <= #1 16'b0;
  else  
    case(chsel)
    3'b000  :  clldata <= #1 test_long[15:0];
    3'b001  :  clldata <= #1 test_long[31:16];
    3'b010  :  clldata <= #1 test_long[47:32];
    3'b011  :  clldata <= #1 test_long[63:48];
    3'b100  :  clldata <= #1 test_long[79:64];
    3'b101  :  clldata <= #1 test_long[95:80];
    3'b110  :  clldata <= #1 test_long[111:96];
    3'b111  :  clldata <= #1 test_long[127:112];
    default :  clldata <= #1 test_long[15:0];
    endcase

always @ (posedge pre_clk or negedge bitslip_reset)
if (~bitslip_reset)
   test_longf  <= #1 16'b0;
else
   test_longf  <= #1 {~rx_out[15 + 16*8],  rx_out[14 + 16*8],
                     ~rx_out[13 + 16*8],  rx_out[12 + 16*8], 
                     ~rx_out[11 + 16*8],  rx_out[10 + 16*8], 
                     ~rx_out[9 + 16*8],   rx_out[8 + 16*8],
                     
                     ~rx_out[7 + 16*8],   rx_out[6 + 16*8], 
                     ~rx_out[5 + 16*8],   rx_out[4 + 16*8], 
                     ~rx_out[3  + 16*8],  rx_out[2 + 16*8], 
                     ~rx_out[1  + 16*8],  rx_out[0 + 16*8]};
 
always @ (posedge pre_clk or negedge gtp_rst)
if (~gtp_rst)
begin
  test_long0    <= #1 16'b0; 
  test_long1    <= #1 16'b0; 
  test_long2    <= #1 16'b0; 
  test_long3    <= #1 16'b0;
  test_long4    <= #1 16'b0; 
  test_long5    <= #1 16'b0; 
  test_long6    <= #1 16'b0; 
  test_long7    <= #1 16'b0;
end
else
begin
  test_long0  <= #1 {~rx_out[15 + 16*0],  ~rx_out[7 + 16*0], 
                     ~rx_out[14 + 16*0],  ~rx_out[6 + 16*0], 
                     ~rx_out[13 + 16*0],  ~rx_out[5 + 16*0], 
                     ~rx_out[12 + 16*0],  ~rx_out[4 + 16*0],
                     
                     ~rx_out[11 + 16*0],  ~rx_out[3 + 16*0], 
                     ~rx_out[10 + 16*0],  ~rx_out[2 + 16*0], 
                     ~rx_out[9  + 16*0],  ~rx_out[1 + 16*0], 
                     ~rx_out[8  + 16*0],  ~rx_out[0 + 16*0]};
                                 
  test_long1  <= #1 {~rx_out[15 + 16*1],  ~rx_out[7 + 16*1], 
                     ~rx_out[14 + 16*1],  ~rx_out[6 + 16*1], 
                     ~rx_out[13 + 16*1],  ~rx_out[5 + 16*1], 
                     ~rx_out[12 + 16*1],  ~rx_out[4 + 16*1],
                     
                     ~rx_out[11 + 16*1],  ~rx_out[3 + 16*1], 
                     ~rx_out[10 + 16*1],  ~rx_out[2 + 16*1], 
                     ~rx_out[9  + 16*1],  ~rx_out[1 + 16*1], 
                     ~rx_out[8  + 16*1],  ~rx_out[0 + 16*1]};
                     
  test_long2  <= #1 {~rx_out[15 + 16*2],  ~rx_out[7 + 16*2], 
                     ~rx_out[14 + 16*2],  ~rx_out[6 + 16*2], 
                     ~rx_out[13 + 16*2],  ~rx_out[5 + 16*2], 
                     ~rx_out[12 + 16*2],  ~rx_out[4 + 16*2],
                     
                     ~rx_out[11 + 16*2],  ~rx_out[3 + 16*2], 
                     ~rx_out[10 + 16*2],  ~rx_out[2 + 16*2], 
                     ~rx_out[9  + 16*2],  ~rx_out[1 + 16*2], 
                     ~rx_out[8  + 16*2],  ~rx_out[0 + 16*2]};
                     
  test_long3  <= #1 {~rx_out[15 + 16*3],  ~rx_out[7 + 16*3], 
                     ~rx_out[14 + 16*3],  ~rx_out[6 + 16*3], 
                     ~rx_out[13 + 16*3],  ~rx_out[5 + 16*3], 
                     ~rx_out[12 + 16*3],  ~rx_out[4 + 16*3],
                     
                     ~rx_out[11 + 16*3],  ~rx_out[3 + 16*3], 
                     ~rx_out[10 + 16*3],  ~rx_out[2 + 16*3], 
                     ~rx_out[9  + 16*3],  ~rx_out[1 + 16*3], 
                     ~rx_out[8  + 16*3],  ~rx_out[0 + 16*3]};
                     
  test_long4  <= #1 {~rx_out[15 + 16*4],  ~rx_out[7 + 16*4], 
                     ~rx_out[14 + 16*4],  ~rx_out[6 + 16*4], 
                     ~rx_out[13 + 16*4],  ~rx_out[5 + 16*4], 
                     ~rx_out[12 + 16*4],  ~rx_out[4 + 16*4],
                     
                     ~rx_out[11 + 16*4],  ~rx_out[3 + 16*4], 
                     ~rx_out[10 + 16*4],  ~rx_out[2 + 16*4], 
                     ~rx_out[9  + 16*4],  ~rx_out[1 + 16*4], 
                     ~rx_out[8  + 16*4],  ~rx_out[0 + 16*4]};
                     
  test_long5  <= #1 {~rx_out[15 + 16*5],  ~rx_out[7 + 16*5], 
                     ~rx_out[14 + 16*5],  ~rx_out[6 + 16*5], 
                     ~rx_out[13 + 16*5],  ~rx_out[5 + 16*5], 
                     ~rx_out[12 + 16*5],  ~rx_out[4 + 16*5],
                     
                     ~rx_out[11 + 16*5],  ~rx_out[3 + 16*5], 
                     ~rx_out[10 + 16*5],  ~rx_out[2 + 16*5], 
                     ~rx_out[9  + 16*5],  ~rx_out[1 + 16*5], 
                     ~rx_out[8  + 16*5],  ~rx_out[0 + 16*5]};
                     
  test_long6  <= #1 {~rx_out[15 + 16*6],  ~rx_out[7 + 16*6], 
                     ~rx_out[14 + 16*6],  ~rx_out[6 + 16*6], 
                     ~rx_out[13 + 16*6],  ~rx_out[5 + 16*6], 
                     ~rx_out[12 + 16*6],  ~rx_out[4 + 16*6],
                     
                     ~rx_out[11 + 16*6],  ~rx_out[3 + 16*6], 
                     ~rx_out[10 + 16*6],  ~rx_out[2 + 16*6], 
                     ~rx_out[9  + 16*6],  ~rx_out[1 + 16*6], 
                     ~rx_out[8  + 16*6],  ~rx_out[0 + 16*6]};
                     
  test_long7  <= #1 {~rx_out[15 + 16*7],  ~rx_out[7 + 16*7], 
                     ~rx_out[14 + 16*7],  ~rx_out[6 + 16*7], 
                     ~rx_out[13 + 16*7],  ~rx_out[5 + 16*7], 
                     ~rx_out[12 + 16*7],  ~rx_out[4 + 16*7],
                     
                     ~rx_out[11 + 16*7],  ~rx_out[3 + 16*7], 
                     ~rx_out[10 + 16*7],  ~rx_out[2 + 16*7], 
                     ~rx_out[9  + 16*7],  ~rx_out[1 + 16*7], 
                     ~rx_out[8  + 16*7],  ~rx_out[0 + 16*7]};
                            
end

wire [7:0] s1_nc0;
wire [7:0] s2_nc0;
wire [7:0] s3_nc0;
wire [7:0] s1d_nc0;
                  
altcnfig altcnfig(
//.clk(cclk),
.clk(sclk),
.rst(pll2_locked),

.syscheck0({(rx_dpa_locked[15] & rx_dpa_locked[14]) ,
            (rx_dpa_locked[13] & rx_dpa_locked[12]) ,
            (rx_dpa_locked[11] & rx_dpa_locked[10]) ,
            (rx_dpa_locked[9]  & rx_dpa_locked[8]) ,
            (rx_dpa_locked[7]  & rx_dpa_locked[6]) ,
            (rx_dpa_locked[5]  & rx_dpa_locked[4]) ,
            (rx_dpa_locked[3]  & rx_dpa_locked[2]) ,
            (rx_dpa_locked[1]  & rx_dpa_locked[0])}),
            
.syscnfig1(s1_nc0),
.syscnfig2(s2_nc0),
.syscnfig3(s3_nc0),

.syscheck4({rx_dpa_locked[17], 
            rx_dpa_locked[16], 
            capture_display,
            collection_display,
            rx_locked_reg, 
            pll2_locked, 
            pll4_locked_reg, 
            bsp_cmp}),
            
.syscnfig1D(s1d_nc0),
            
.syscheck8(8'h08),           
.syscheckC(8'h0C),
.syscheck10(8'h10),
.syscheck14(8'h14),
.syscheck18(8'h18),
.syscheck1C(8'h1C)
);

            
assign #1 memsize             = s1_nc0[7:4];
assign #1 chsel               = s1_nc0[3:1];

assign #1 capture_clr_start   = s2_nc0[0];
assign #1 data_clr_collection = s3_nc0[0];
                                                          
assign #1 MPI2_RESET          = ~s1d_nc0[0];

                                           
                                         
reg [55:0] BUFSIZE_DSPLY;
always @ (memsize)
  casex(memsize)
  // MEMSIZE_64KSPS
  //4'b1101 :  begin BUFFER_ADDR_CNT <=  #1 13'h1fff;
  //                 BUFSIZE_DSPLY   <=  #1 {_sp, _6, _4, _Kc, _Sc, _Pc, _s};
  //           end
  // MEMSIZE_32KSPS
  //4'b1100 :  begin BUFFER_ADDR_CNT <=  #1 12'h0fff;
  //                 BUFSIZE_DSPLY   <=  #1 {_sp, _3, _2, _Kc, _Sc, _Pc, _s};
  //           end
  // MEMSIZE_16KSPS
  4'b1011 :  begin BUFFER_ADDR_CNT <=  #1 11'h7ff;
                   BUFSIZE_DSPLY   <=  #1 {_sp, _1, _6, _Kc, _Sc, _Pc, _s};
             end  
  // MEMSIZE_8KSPS
  4'b1010 :  begin BUFFER_ADDR_CNT <=  #1 11'h3ff;
                   BUFSIZE_DSPLY   <=  #1 {_sp, _sp, _8, _Kc, _Sc, _Pc, _s};
             end
  // MEMSIZE_4KSPS
  4'b1001 :  begin BUFFER_ADDR_CNT <=  #1 11'h1ff;
                   BUFSIZE_DSPLY   <=  #1 {_sp, _sp, _4, _Kc, _Sc, _Pc, _s};
             end  
  // MEMSIZE_2KSPS
  4'b1000 :  begin BUFFER_ADDR_CNT <=  #1 11'h0ff;
                   BUFSIZE_DSPLY   <=  #1 {_sp, _sp, _2, _Kc, _Sc, _Pc, _s};
             end  
  // MEMSIZE_1KSPS
  4'b0111 :  begin BUFFER_ADDR_CNT <=  #1 11'h07f;
                   BUFSIZE_DSPLY   <=  #1 {_sp, _sp, _1, _Kc, _Sc, _Pc, _s};
             end  
  // MEMSIZE_512SPS
  4'b0110 :  begin BUFFER_ADDR_CNT <=  #1 11'h03f;
                   BUFSIZE_DSPLY   <=  #1 {_sp, _5, _1, _2, _Sc, _Pc, _s};
             end  
  // MEMSIZE_256SPS
  4'b0101 :  begin BUFFER_ADDR_CNT <=  #1 11'h01f;
                   BUFSIZE_DSPLY   <=  #1 {_sp, _2, _5, _6, _Sc, _Pc, _s};
             end  
  // MEMSIZE_128SPS
  4'b0100 :  begin BUFFER_ADDR_CNT <=  #1 11'h00f;
                   BUFSIZE_DSPLY   <=  #1 {_sp, _1, _2, _8, _Sc, _Pc, _s};
             end  
  // MEMSIZE_64SPS
  4'b0011 :  begin BUFFER_ADDR_CNT <=  #1 11'h007;
                   BUFSIZE_DSPLY   <=  #1 {_sp, _sp, _6, _4, _Sc, _Pc, _s};
             end  
  // MEMSIZE_32SPS
  4'b0010 :  begin BUFFER_ADDR_CNT <=  #1 11'h003;
                   BUFSIZE_DSPLY   <=  #1 {_sp, _sp, _3, _2, _Sc, _Pc, _s};
             end  
  // MEMSIZE_16SPS
  4'b0001 :  begin BUFFER_ADDR_CNT <=  #1 11'h001;
                   BUFSIZE_DSPLY   <=  #1 {_sp, _sp, _1, _6, _Sc, _Pc, _s};
             end  
  // MEMSIZE_8SPS
  4'b0000 :  begin BUFFER_ADDR_CNT <=  #1 11'h000;
                   BUFSIZE_DSPLY   <=  #1 {_sp, _sp, _sp, _8, _Sc, _Pc, _s};
             end  
  // MEMSIZE_1KSPS
  default :  begin BUFFER_ADDR_CNT <=  #1 11'h07f;
                   BUFSIZE_DSPLY   <=  #1 {_sp, _sp, _1, _Kc, _Sc, _Pc, _s};
             end   
  endcase


/*  
reg [15:0] MODE_DSPLY;  
always @ (MODE_JMP)
  if(MODE_JMP)
    MODE_DSPLY <= #1 {_Sc, _F};
  else
    MODE_DSPLY <= #1 {_M, _F};
*/
    
reg [7:0] CHSEL_DSPLY;
always @ (chsel)
  casex(chsel)
  3'b000  :  CHSEL_DSPLY <= #1 _0;
  3'b001  :  CHSEL_DSPLY <= #1 _1;
  3'b010  :  CHSEL_DSPLY <= #1 _2;
  3'b011  :  CHSEL_DSPLY <= #1 _3;
  3'b100  :  CHSEL_DSPLY <= #1 _4;
  3'b101  :  CHSEL_DSPLY <= #1 _5;
  3'b110  :  CHSEL_DSPLY <= #1 _6;
  3'b111  :  CHSEL_DSPLY <= #1 _7;
  default :  CHSEL_DSPLY <= #1 _0;
  endcase
 
         
reg [23:0] STATUS_DSPLY;  
always @ (gtp_rst)
  if(gtp_rst)
    STATUS_DSPLY <= #1 {_Rc, _Uc, _Nc};
  else
    STATUS_DSPLY <= #1 {_Rc, _Sc, _Tc};
                         
    
  
wire rsgen;
wire rwgen;
wire egen;
wire [7:0]lcdgen;
 
lcdrw_ctrl lcdrw_ctrl(
.rst(pll2_locked),//fpga rst status check gyu
.clk(sclk),

.rsin(rsgen),
.rwin(rwgen),
.ein(egen),
.lcdin(lcdgen),

.rsout(lcd_d_cn),
.rwout(lcd_wen),
.eout(lcd_csn),

.lcdda(lcd_data)
  );
  
lcdfl_ctrl lcdfl_ctrl(
.rst(pll2_locked),//fpga rst status check gyu
.clk(sclk),

.rsgen(rsgen),
.rwgen(rwgen),
.egen(egen),
.lcdgen(lcdgen),

//.lcd_up_line({STATUS_TITLE_DSPLY, STATUS_DSPLY, _sp, MODE_TITLE_DSPLY, MODE_DSPLY}),
.lcd_up_line({STATUS_TITLE_DSPLY, STATUS_DSPLY, _sp, CHANNEL_TITLE_DSPLY, CHSEL_DSPLY}),
              
.lcd_dn_line({BUFSIZE_TITLE_DSPLY, BUFSIZE_DSPLY}),

.lcd_busy(lcd_data[7])
  );

endmodule



